@@ -51,10 +51,11 @@ import nokogiri.internals.c14n.Canonicalizer;
  * @author Yoko Harada <yokolet@gmail.com>
  * @author John Shahid <jvshahid@gmail.com>
  */
-
 @JRubyClass(name = "Nokogiri::XML::Document", parent = "Nokogiri::XML::Node")
 public class XmlDocument extends XmlNode
 {
+  private static final long serialVersionUID = 1L;
+
   private NokogiriNamespaceCache nsCache;
 
   /* UserData keys for storing extra info in the document node. */
@@ -600,7 +601,7 @@ public class XmlDocument extends XmlNode
   public static IRubyObject
   wrap(ThreadContext context, IRubyObject klass, IRubyObject arg)
   {
-    XmlDocument xmlDocument = new XmlDocument(context.runtime, (RubyClass) klass, (Document) arg.toJava(Document.class));
+    XmlDocument xmlDocument = new XmlDocument(context.runtime, (RubyClass) klass, arg.toJava(Document.class));
     Helpers.invoke(context, xmlDocument, "initialize");
     return xmlDocument;
   }
@@ -680,11 +681,9 @@ public class XmlDocument extends XmlNode
         result = canonicalizer.canonicalizeSubtree(startingNode.getNode(), inclusive_namespace, filter);
       }
       return RubyString.newString(context.runtime, new ByteList(result, UTF8Encoding.INSTANCE));
-    } catch (CanonicalizationException e) {
-      // TODO Auto-generated catch block
-      e.printStackTrace();
+    } catch (Exception e) {
+      throw context.getRuntime().newRuntimeError(e.getMessage());
     }
-    return context.nil;
   }
 
   private XmlNode
