@@ -28,6 +28,7 @@ import org.w3c.dom.NodeList;
 @JRubyClass(name = "Nokogiri::XML::NodeSet")
 public class XmlNodeSet extends RubyObject implements NodeList
 {
+  private static final long serialVersionUID = 1L;
 
   IRubyObject[] nodes;
 
@@ -350,12 +351,14 @@ public class XmlNodeSet extends RubyObject implements NodeList
     if (indexOrRange instanceof RubyFixnum) {
       return slice(context, ((RubyFixnum) indexOrRange).getIntValue());
     }
-
-    int[] begLen = new int[2];
-    rangeBeginLength(context, indexOrRange, nodes.length, begLen);
-    int min = begLen[0];
-    int max = begLen[1];
-    return subseq(context, min, max - min);
+    if (indexOrRange instanceof RubyRange) {
+      int[] begLen = new int[2];
+      rangeBeginLength(context, indexOrRange, nodes.length, begLen);
+      int min = begLen[0];
+      int max = begLen[1];
+      return subseq(context, min, max - min);
+    }
+    throw context.runtime.newTypeError("index must be an Integer or a Range");
   }
 
   IRubyObject
@@ -407,7 +410,7 @@ public class XmlNodeSet extends RubyObject implements NodeList
   }
 
   @JRubyMethod(name = {"to_a", "to_ary"})
-  public RubyArray
+  public RubyArray<?>
   to_a(ThreadContext context)
   {
     return context.runtime.newArrayNoCopy(nodes);
