@@ -25,7 +25,7 @@ import org.w3c.dom.Document;
 import org.w3c.dom.Node;
 import org.w3c.dom.NodeList;
 
-import nokogiri.HtmlDocument;
+import nokogiri.Html4Document;
 import nokogiri.NokogiriService;
 import nokogiri.XmlAttr;
 import nokogiri.XmlCdata;
@@ -89,7 +89,7 @@ public class NokogiriHelpers
     if (node == null) { return runtime.getNil(); }
     if (node.getNodeType() == Node.ATTRIBUTE_NODE && isNamespace(node.getNodeName())) {
       XmlDocument xmlDocument = (XmlDocument) node.getOwnerDocument().getUserData(CACHED_NODE);
-      if (!(xmlDocument instanceof HtmlDocument)) {
+      if (!(xmlDocument instanceof Html4Document)) {
         String prefix = getLocalNameForNamespace(((Attr) node).getName(), null);
         String href = ((Attr) node).getValue();
         XmlNamespace xmlNamespace = xmlDocument.getNamespaceCache().get(prefix, href);
@@ -116,59 +116,59 @@ public class NokogiriHelpers
     if (node == null) { return runtime.getNil(); }
     // this is slow; need a way to cache nokogiri classes/modules somewhere
     switch (node.getNodeType()) {
-    case Node.ELEMENT_NODE:
-      XmlElement xmlElement = (XmlElement) NokogiriService.XML_ELEMENT_ALLOCATOR.allocate(runtime, getNokogiriClass(runtime,
-                              "Nokogiri::XML::Element"));
-      xmlElement.setNode(runtime, node);
-      return xmlElement;
-    case Node.ATTRIBUTE_NODE:
-      XmlAttr xmlAttr = (XmlAttr) NokogiriService.XML_ATTR_ALLOCATOR.allocate(runtime, getNokogiriClass(runtime,
-                        "Nokogiri::XML::Attr"));
-      xmlAttr.setNode(runtime, node);
-      return xmlAttr;
-    case Node.TEXT_NODE:
-      XmlText xmlText = (XmlText) NokogiriService.XML_TEXT_ALLOCATOR.allocate(runtime, getNokogiriClass(runtime,
-                        "Nokogiri::XML::Text"));
-      xmlText.setNode(runtime, node);
-      return xmlText;
-    case Node.COMMENT_NODE:
-      XmlComment xmlComment = (XmlComment) NokogiriService.XML_COMMENT_ALLOCATOR.allocate(runtime, getNokogiriClass(runtime,
-                              "Nokogiri::XML::Comment"));
-      xmlComment.setNode(runtime, node);
-      return xmlComment;
-    case Node.ENTITY_NODE:
-      return new XmlNode(runtime, getNokogiriClass(runtime, "Nokogiri::XML::EntityDecl"), node);
-    case Node.ENTITY_REFERENCE_NODE:
-      XmlEntityReference xmlEntityRef = (XmlEntityReference) NokogiriService.XML_ENTITY_REFERENCE_ALLOCATOR.allocate(runtime,
-                                        getNokogiriClass(runtime, "Nokogiri::XML::EntityReference"));
-      xmlEntityRef.setNode(runtime, node);
-      return xmlEntityRef;
-    case Node.PROCESSING_INSTRUCTION_NODE:
-      XmlProcessingInstruction xmlProcessingInstruction = (XmlProcessingInstruction)
-          NokogiriService.XML_PROCESSING_INSTRUCTION_ALLOCATOR.allocate(runtime, getNokogiriClass(runtime,
-              "Nokogiri::XML::ProcessingInstruction"));
-      xmlProcessingInstruction.setNode(runtime, node);
-      return xmlProcessingInstruction;
-    case Node.CDATA_SECTION_NODE:
-      XmlCdata xmlCdata = (XmlCdata) NokogiriService.XML_CDATA_ALLOCATOR.allocate(runtime, getNokogiriClass(runtime,
-                          "Nokogiri::XML::CDATA"));
-      xmlCdata.setNode(runtime, node);
-      return xmlCdata;
-    case Node.DOCUMENT_NODE:
-      XmlDocument xmlDocument = (XmlDocument) NokogiriService.XML_DOCUMENT_ALLOCATOR.allocate(runtime,
-                                getNokogiriClass(runtime, "Nokogiri::XML::Document"));
-      xmlDocument.setDocumentNode(runtime, (Document) node);
-      return xmlDocument;
-    case Node.DOCUMENT_TYPE_NODE:
-      XmlDtd xmlDtd = (XmlDtd) NokogiriService.XML_DTD_ALLOCATOR.allocate(runtime, getNokogiriClass(runtime,
-                      "Nokogiri::XML::DTD"));
-      xmlDtd.setNode(runtime, node);
-      return xmlDtd;
-    default:
-      XmlNode xmlNode = (XmlNode) NokogiriService.XML_NODE_ALLOCATOR.allocate(runtime, getNokogiriClass(runtime,
-                        "Nokogiri::XML::Node"));
-      xmlNode.setNode(runtime, node);
-      return xmlNode;
+      case Node.ELEMENT_NODE:
+        XmlElement xmlElement = (XmlElement) NokogiriService.XML_ELEMENT_ALLOCATOR.allocate(runtime, getNokogiriClass(runtime,
+                                "Nokogiri::XML::Element"));
+        xmlElement.setNode(runtime, node);
+        return xmlElement;
+      case Node.ATTRIBUTE_NODE:
+        XmlAttr xmlAttr = (XmlAttr) NokogiriService.XML_ATTR_ALLOCATOR.allocate(runtime, getNokogiriClass(runtime,
+                          "Nokogiri::XML::Attr"));
+        xmlAttr.setNode(runtime, node);
+        return xmlAttr;
+      case Node.TEXT_NODE:
+        XmlText xmlText = (XmlText) NokogiriService.XML_TEXT_ALLOCATOR.allocate(runtime, getNokogiriClass(runtime,
+                          "Nokogiri::XML::Text"));
+        xmlText.setNode(runtime, node);
+        return xmlText;
+      case Node.COMMENT_NODE:
+        XmlComment xmlComment = (XmlComment) NokogiriService.XML_COMMENT_ALLOCATOR.allocate(runtime, getNokogiriClass(runtime,
+                                "Nokogiri::XML::Comment"));
+        xmlComment.setNode(runtime, node);
+        return xmlComment;
+      case Node.ENTITY_NODE:
+        return new XmlNode(runtime, getNokogiriClass(runtime, "Nokogiri::XML::EntityDecl"), node);
+      case Node.ENTITY_REFERENCE_NODE:
+        XmlEntityReference xmlEntityRef = (XmlEntityReference) NokogiriService.XML_ENTITY_REFERENCE_ALLOCATOR.allocate(runtime,
+                                          getNokogiriClass(runtime, "Nokogiri::XML::EntityReference"));
+        xmlEntityRef.setNode(runtime, node);
+        return xmlEntityRef;
+      case Node.PROCESSING_INSTRUCTION_NODE:
+        XmlProcessingInstruction xmlProcessingInstruction = (XmlProcessingInstruction)
+            NokogiriService.XML_PROCESSING_INSTRUCTION_ALLOCATOR.allocate(runtime, getNokogiriClass(runtime,
+                "Nokogiri::XML::ProcessingInstruction"));
+        xmlProcessingInstruction.setNode(runtime, node);
+        return xmlProcessingInstruction;
+      case Node.CDATA_SECTION_NODE:
+        XmlCdata xmlCdata = (XmlCdata) NokogiriService.XML_CDATA_ALLOCATOR.allocate(runtime, getNokogiriClass(runtime,
+                            "Nokogiri::XML::CDATA"));
+        xmlCdata.setNode(runtime, node);
+        return xmlCdata;
+      case Node.DOCUMENT_NODE:
+        XmlDocument xmlDocument = (XmlDocument) NokogiriService.XML_DOCUMENT_ALLOCATOR.allocate(runtime,
+                                  getNokogiriClass(runtime, "Nokogiri::XML::Document"));
+        xmlDocument.setDocumentNode(runtime, (Document) node);
+        return xmlDocument;
+      case Node.DOCUMENT_TYPE_NODE:
+        XmlDtd xmlDtd = (XmlDtd) NokogiriService.XML_DTD_ALLOCATOR.allocate(runtime, getNokogiriClass(runtime,
+                        "Nokogiri::XML::DTD"));
+        xmlDtd.setNode(runtime, node);
+        return xmlDtd;
+      default:
+        XmlNode xmlNode = (XmlNode) NokogiriService.XML_NODE_ALLOCATOR.allocate(runtime, getNokogiriClass(runtime,
+                          "Nokogiri::XML::Node"));
+        xmlNode.setNode(runtime, node);
+        return xmlNode;
     }
   }
 
@@ -645,10 +645,10 @@ public class NokogiriHelpers
     return result;
   }
 
-  public static RubyArray
+  public static RubyArray<?>
   nodeArrayToRubyArray(Ruby ruby, Node[] nodes)
   {
-    RubyArray n = RubyArray.newArray(ruby, nodes.length);
+    RubyArray<?> n = RubyArray.newArray(ruby, nodes.length);
     for (int i = 0; i < nodes.length; i++) {
       n.append(NokogiriHelpers.getCachedNodeOrCreate(ruby, nodes[i]));
     }
@@ -723,8 +723,8 @@ public class NokogiriHelpers
   public static CharSequence
   convertEncodingByNKFIfNecessary(ThreadContext context, XmlDocument doc, CharSequence str)
   {
-    if (!(doc instanceof HtmlDocument)) { return str; }
-    String parsed_encoding = ((HtmlDocument)doc).getPraedEncoding();
+    if (!(doc instanceof Html4Document)) { return str; }
+    String parsed_encoding = ((Html4Document)doc).getPraedEncoding();
     if (parsed_encoding == null) { return str; }
     String ruby_encoding = rubyStringToString(doc.getEncoding());
     if (ruby_encoding == null) { return str; }
@@ -754,10 +754,9 @@ public class NokogiriHelpers
     else if (NokogiriHelpers.EUC_JP.compareTo(encoding) == 0) { opt = _Ew; }
     else { opt = _Ww; } // should not come here. should be treated before this method.
 
-    Class nkfClass;
+    Class<?> nkfClass;
     try {
-      // JRuby 1.7 and later
-      nkfClass = runtime.getClassLoader().loadClass("org.jruby.ext.nkf.RubyNKF");
+      nkfClass = Ruby.getClassLoader().loadClass("org.jruby.ext.nkf.RubyNKF");
     } catch (ClassNotFoundException e1) {
       return str;
     }
