@@ -74,6 +74,7 @@ public class SaveContextVisitor
   public static final int AS_XML = 32;
   public static final int AS_HTML = 64;
   public static final int AS_BUILDER = 128;
+  public static final int DEFAULT_HTML = NO_DECL | NO_EMPTY | AS_HTML;
 
   public static final int CANONICAL = 1;
   public static final int INCL_NS = 2;
@@ -807,16 +808,13 @@ public class SaveContextVisitor
   }
 
   private boolean
-  isHtmlScript(Text text)
+  isCDATA(Text text)
   {
-    return htmlDoc && text.getParentNode().getNodeName().equals("script");
+    Node parentNode = text.getParentNode();
+    return htmlDoc && parentNode != null && (parentNode.getNodeName().equals("style")
+           || parentNode.getNodeName().equals("script"));
   }
 
-  private boolean
-  isHtmlStyle(Text text)
-  {
-    return htmlDoc && text.getParentNode().getNodeName().equals("style");
-  }
 
   public boolean
   enter(Text text)
@@ -830,7 +828,7 @@ public class SaveContextVisitor
       }
     }
 
-    if (shouldEncode(text) && !isHtmlScript(text) && !isHtmlStyle(text)) {
+    if (shouldEncode(text) && !isCDATA(text)) {
       textContent = encodeJavaString(textContent);
     }
 
