@@ -19,22 +19,29 @@
 package nokogiri.internals.c14n;
 
 
-public class Canonicalizer20010315ExclOmitComments extends Canonicalizer20010315Excl {
+public class Canonicalizer20010315ExclOmitComments extends Canonicalizer20010315Excl
+{
 
-    /**
-     * 
-     */
-    public Canonicalizer20010315ExclOmitComments() {
-        super(false);
-    }
+  /**
+   *
+   */
+  public
+  Canonicalizer20010315ExclOmitComments()
+  {
+    super(false);
+  }
 
-    /** @inheritDoc */
-    public final String engineGetURI() {
-        return Canonicalizer.ALGO_ID_C14N_EXCL_OMIT_COMMENTS;
-    }
+  /** @inheritDoc */
+  public final String
+  engineGetURI()
+  {
+    return Canonicalizer.ALGO_ID_C14N_EXCL_OMIT_COMMENTS;
+  }
 
-    /** @inheritDoc */
-    public final boolean engineGetIncludeComments() {
-        return false;
-    }
+  /** @inheritDoc */
+  public final boolean
+  engineGetIncludeComments()
+  {
+    return false;
+  }
 }
