@@ -63,7 +63,7 @@ module Nokogiri
         elsif Nokogiri.uses_libxml?
           assert_equal(61, sub_elements.length)
         else
-          refute_empty(sub_elements)
+          assert_equal(105, sub_elements.length)
         end
       end
 
@@ -72,7 +72,7 @@ module Nokogiri
       end
 
       def test_null_default_sub_element
-        doc = Nokogiri::HTML("foo")
+        doc = Nokogiri::HTML4("foo")
         doc.root.description.default_sub_element
       end
 
