@@ -164,7 +164,7 @@ class TestHtml5API < Nokogiri::TestCase
     span = doc.at_xpath("/html/body/form/span")
     refute_nil(span)
     frag = span.fragment("<form>Nested forms should be ignored</form>")
-    assert(frag.is_a?(Nokogiri::HTML5::DocumentFragment))
+    assert_kind_of(Nokogiri::HTML5::DocumentFragment, frag)
     assert_equal(1, frag.children.length)
     nested_form = frag.at_xpath("form")
     assert_nil(nested_form)
@@ -176,7 +176,7 @@ class TestHtml5API < Nokogiri::TestCase
     span = doc.at_xpath("/html/body/span")
     refute_nil(span)
     frag = span.fragment("<form><span>Form should not be ignored</span></form>")
-    assert(frag.is_a?(Nokogiri::HTML5::DocumentFragment))
+    assert_kind_of(Nokogiri::HTML5::DocumentFragment, frag)
     assert_equal(1, frag.children.length)
     form = frag.at_xpath("form")
     refute_nil(form)
@@ -185,7 +185,7 @@ class TestHtml5API < Nokogiri::TestCase
   def test_empty_fragment
     doc = Nokogiri.HTML5("<!DOCTYPE html><body>")
     frag = doc.fragment
-    assert(frag.is_a?(Nokogiri::HTML5::DocumentFragment))
+    assert_kind_of(Nokogiri::HTML5::DocumentFragment, frag)
     assert_empty(frag.children)
   end
 
@@ -238,6 +238,21 @@ class TestHtml5API < Nokogiri::TestCase
     assert_equal("select", el.parent.parent.name)
   end
 
+  def test_parse_in_context_of_foreign_namespace
+    # https://github.com/sparklemotion/nokogiri/issues/3112
+    # https://gitlab.gnome.org/GNOME/libxml2/-/issues/672
+    # released upstream in v2.12.5
+    skip if Nokogiri.uses_libxml?(["~> 2.12.0", "< 2.12.5"])
+
+    doc = Nokogiri::HTML5::Document.parse("<html><body><math>")
+    math = doc.at_css("math")
+
+    nodes = math.parse("mrow") # segfaults in libxml 2.12 before 95f2a174
+
+    assert_kind_of(Nokogiri::XML::NodeSet, nodes)
+    assert_equal(1, nodes.length)
+  end
+
   describe Nokogiri::HTML5::Document do
     describe "#fragment" do
       it "parses text nodes in a `body` context" do
