@@ -143,7 +143,7 @@ module Nokogiri
         def test_parse_error_list
           error_count = xml.errors.length
           xml.root.parse("<hello>")
-          assert(error_count < xml.errors.length, "errors should have increased")
+          assert_operator(error_count, :<, xml.errors.length, "errors should have increased")
         end
 
         def test_parse_error_on_fragment_with_empty_document
@@ -547,7 +547,7 @@ module Nokogiri
           node = xml.at("address")
           ns = node.add_namespace(nil, "http://tenderlovemaking.com")
           ns2 = node.add_namespace(nil, "http://tenderlovemaking.com")
-          assert_equal(ns.object_id, ns2.object_id)
+          assert_same(ns, ns2)
         end
 
         def test_add_multiple_namespaces
@@ -1328,18 +1328,9 @@ module Nokogiri
           assert_equal("fr", subject.lang)
         end
 
-        def test_text_node_robustness_gh1426
-          skip("only run if NOKOGIRI_GC is set") unless ENV["NOKOGIRI_GC"]
-          skip_unless_libxml2("No need to test libxml-ruby workarounds on JRuby")
-          # notably, the original bug report was about libxml-ruby interactions
-          # this test should blow up under valgrind if we regress on libxml-ruby workarounds
-          # side note: this was fixed in libxml-ruby 2.9.0 by https://github.com/xml4r/libxml-ruby/pull/119
-          message = "<section><h2>BOOM!</h2></section>"
-          10_000.times do
-            node = Nokogiri::HTML4::DocumentFragment.parse(message).at_css("h2")
-            node.add_previous_sibling(Nokogiri::XML::Text.new("before", node.document))
-            node.add_next_sibling(Nokogiri::XML::Text.new("after", node.document))
-          end
+        # issue 2559
+        def test_serialize_unparented_node
+          assert_equal("asdf", Nokogiri::HTML4::Document.parse("<div></div>").create_text_node("asdf").to_s)
         end
 
         describe "#wrap" do
@@ -1506,7 +1497,7 @@ module Nokogiri
             if Nokogiri.uses_libxml?
               doc = Nokogiri::XML(xml, &:nobig_lines)
               node = doc.at_css("x")
-              assert_operator(node.line, :==, max_short_int)
+              assert_equal(node.line, max_short_int)
             end
 
             doc = Nokogiri::XML(xml)
