@@ -173,8 +173,8 @@ module Nokogiri
             set = xml.xpath("//staff")
 
             [
-              [:xpath,  "//*[awesome(.)]"],
-              [:search, "//*[awesome(.)]"],
+              [:xpath,  "//*[nokogiri:awesome(.)]"],
+              [:search, "//*[nokogiri:awesome(.)]"],
               [:css,    "*:awesome"],
               [:search, "*:awesome"],
             ].each do |method, query|
@@ -185,19 +185,28 @@ module Nokogiri
               end.new
               custom_employees = set.send(method, query, callback_handler)
 
-              assert_equal(xml.xpath("//employee"), custom_employees,
-                "using #{method} with custom selector '#{query}'")
+              assert_equal(
+                xml.xpath("//employee"),
+                custom_employees,
+                "using #{method} with custom selector '#{query}'",
+              )
             end
           end
 
           it "with variable bindings" do
             set = xml.xpath("//staff")
 
-            assert_equal(4, set.xpath("//address[@domestic=$value]", nil, value: "Yes").length,
-              "using #xpath with variable binding")
+            assert_equal(
+              4,
+              set.xpath("//address[@domestic=$value]", nil, value: "Yes").length,
+              "using #xpath with variable binding",
+            )
 
-            assert_equal(4, set.search("//address[@domestic=$value]", nil, value: "Yes").length,
-              "using #search with variable binding")
+            assert_equal(
+              4,
+              set.search("//address[@domestic=$value]", nil, value: "Yes").length,
+              "using #search with variable binding",
+            )
           end
 
           it "context search returns itself" do
@@ -275,22 +284,22 @@ module Nokogiri
             assert(node_set_one = xml.xpath("//employee"))
             assert(node_set_two = xml.xpath("//employee"))
 
-            refute_equal(node_set_one.object_id, node_set_two.object_id)
+            refute_same(node_set_one, node_set_two)
             refute_same(node_set_one, node_set_two)
 
-            assert_equal(node_set_one, node_set_two)
+            assert_operator(node_set_one, :==, node_set_two) # rubocop:disable Minitest/AssertEqual
           end
 
           it "handles comparison to a string" do
             node_set_one = xml.xpath("//employee")
-            refute(node_set_one == "asdfadsf")
+            refute_operator(node_set_one, :==, "asdfadsf") # rubocop:disable Minitest/RefuteEqual
           end
 
           it "returns false if same elements are out of order" do
             one = xml.xpath("//employee")
             two = xml.xpath("//employee")
             two.push(two.shift)
-            refute_equal(one, two)
+            refute_operator(one, :==, two) # rubocop:disable Minitest/RefuteEqual
           end
 
           it "returns false if one is a subset of the other" do
@@ -298,8 +307,8 @@ module Nokogiri
             node_set_two = xml.xpath("//employee")
             node_set_two.delete(node_set_two.first)
 
-            refute(node_set_one == node_set_two)
-            refute(node_set_two == node_set_one)
+            refute_operator(node_set_one, :==, node_set_two) # rubocop:disable Minitest/RefuteEqual
+            refute_operator(node_set_two, :==, node_set_one) # rubocop:disable Minitest/RefuteEqual
           end
         end
 
@@ -600,11 +609,15 @@ module Nokogiri
           end
 
           it "preserves document structure" do
-            assert_equal("employeeId",
-              xml.at_xpath("//employee").children.detect { |j| !j.text? }.name)
+            assert_equal(
+              "employeeId",
+              xml.at_xpath("//employee").children.detect { |j| !j.text? }.name,
+            )
             xml.xpath("//employeeId[text()='EMP0001']").wrap("<wrapper/>")
-            assert_equal("wrapper",
-              xml.at_xpath("//employee").children.detect { |j| !j.text? }.name)
+            assert_equal(
+              "wrapper",
+              xml.at_xpath("//employee").children.detect { |j| !j.text? }.name,
+            )
           end
         end
 
@@ -792,6 +805,7 @@ module Nokogiri
           it "returns an enumerator given no block" do
             skip("enumerators confuse valgrind") if i_am_running_in_valgrind
             skip("enumerators confuse ASan") if i_am_running_with_asan
+            skip("https://bugs.ruby-lang.org/issues/20085") if RUBY_DESCRIPTION.include?("aarch64") && RUBY_VERSION == "3.3.0"
 
             employees = xml.search("//employee")
             enum = employees.each
@@ -821,8 +835,10 @@ module Nokogiri
           employees = xml.search("//employee")
           inspected = employees.inspect
 
-          assert_equal("[#{employees.map(&:inspect).join(", ")}]",
-            inspected)
+          assert_equal(
+            "[#{employees.map(&:inspect).join(", ")}]",
+            inspected,
+          )
         end
 
         it "should_not_splode_when_accessing_namespace_declarations_in_a_node_set" do
